package com.topcoder.db.generation.xsd2ddl;

/**
 * The ParseException exception is used to signal an implementation specific parsing exception.
 * It is thrown by the DefinitionParser interface implementations.
 * 
 * The exception can be used directly if an implementation specific error condition is encountered
 * or it can be used to wrap another exception that resulted from the actions done by the implementation.
 * 
 * The exception can store a text message and can wrap an eventual cause exception.
 *
 * Thread safety:
 * 
 * The class is immutable so it is thread safe.
 *
 * @author TCSDESIGNER
 */
public class ParseException extends ProcessingException {

    /**
     * Create a new instance with the given error message.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message.
     *
     * @param message the message describing the exception
     *
     * @throws IllegalArgumentException if the message is null
     */
    public ParseException(String message) throws IllegalArgumentException {
        super(message);
    }

    /**
     * Create a new instance with the given error message and the given cause.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message and cause.
     *
     * @param message the message describing the exception
     * @param cause the cause of the exception
     *
     * @throws IllegalArgumentException if the message is null or if cause is null
     */
    public ParseException(String message, Throwable cause) throws IllegalArgumentException {
        super(message, cause);
    }

}
