package com.topcoder.db.generation.xsd2ddl.data;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * This is a data class that models a database.
 * This is just a container of tables.
 * However, in future versions it could evolve to contain other entities as well
 * so it is defined for extensibility even if a Table[] array could do the same job.
 * 
 * The table list has appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class Database {
	
    /**
     * Stores the tables contained in this database.
     * It is a table name (String) to Table mapping.
     * Cannot be null, have null keys or values.
     * Initialized with LinkedHashMap (because we need order).
     * Used by all the table accessor methods.
     */
	private Map tables = new LinkedHashMap();
    
    /**
     * Default constructor.
     * Doesn't do anything.
     */
	public Database() {
    }

	/**
	 * Adds a table to the list of tables.
	 * If a table with the same name exists, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Checks the mapping for table.getTableName() and adds it to the mapping is not present.
	 * 
	 * @param table the table to add
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean addTable(Table table) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Removes a table from the list of tables.
	 * If the table doesn't exist, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Delete the table.getTableName() entry.
	 * 
	 * @param table the table to remove
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean removeTable(Table table) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears all tables.
	 * 
	 * Implementation details:
	 * 
	 * Clear the map.
	 */
	public void clearTables() {
	}
	
	/**
	 * Gets all tables.
	 * 
	 * Implementation details:
	 * 
	 * Make an array from the value set and return it.
	 * 
	 * @return the contained tables
	 */
	public Table[] getTables() {
		return null;
	}
	
	/**
	 * Gets a table by name.
	 * If no such table exist, null is returned
	 * 
	 * Implementation details:
	 * 
	 * Finds in the mapping the value for the name key.
	 * 
	 * @param name the table name
	 * @return the table with the given name or null if not found
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Table getTable(String name) throws IllegalArgumentException {
		return null;
	}

}
