package com.topcoder.db.generation.xsd2ddl.data;

/**
 * This is a data class that models a field.
 * The field has a name, a field type and a nullable indicator.
 * 
 * All these attributes have appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class Field {
	
	/**
	 * The name of the field.
	 * Cannot be null, empty or all spaces (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private String fieldName = null;
	
	/**
	 * The type of the field.
	 * Cannot be null (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private FieldType fieldType = null;

	/**
	 * Indicates whether the field is nullable or not.
	 * Used by its setter and getter. Defaults to true.
	 */
	private boolean nullable = true;
	
    /**
     * Default constructor.
     * Doesn't do anything.
     */
	public Field() {
	}

	/**
	 * Gets the field name (fieldName field)
	 * 
	 * @return the field name
	 */
	public String getFieldName() {
		return fieldName;
	}

	/**
	 * Sets the field name (fieldName field)
	 * 
	 * @param fieldName the field name to set
	 * 
	 * @throws IllegalArgumentException if argument is null, empty or all spaces
	 */
	public void setFieldName(String fieldName) throws IllegalArgumentException {
		this.fieldName = fieldName;
	}

	/**
	 * Gets the field type (fieldType field)
	 * 
	 * @return the field type
	 */
	public FieldType getFieldType() {
		return fieldType;
	}

	/**
	 * Sets the field type (fieldType field)
	 * 
	 * @param fieldType the field type to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument is empty or all spaces
	 */
	public void setFieldType(FieldType fieldType) throws IllegalArgumentException {
		this.fieldType = fieldType;
	}

	/**
	 * Returns whether the field can contain nulls (nullable attribute).
	 * 
	 * @return if nullable or not
	 */
	public boolean isNullable() {
		return nullable;
	}

	/**
	 * Sets whether the field can contain nulls (nullable attribute).
	 * 
	 * @param nullable the nullable value to set
	 */
	public void setNullable(boolean nullable) {
		this.nullable = nullable;
	}

}
