package com.topcoder.db.generation.xsd2ddl.data;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * This is a data class that models a primary or foreign key.
 * The key has a name, the fields the key is made of, and for foreign keys 
 * it has the referred table and primary key.
 * 
 * All these attributes have appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class Key {

	/**
	 * The name of the key.
	 * Cannot be null, empty or all spaces (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private String keyName = null;
	
	/**
	 * The referred table (for foreign keys).
	 * Can be null.
	 * Used by its setter and getter. 
	 */
	private Table referredTable = null;

	/**
	 * The referred key (for foreign keys).
	 * Can be null.
	 * Used by its setter and getter. 
	 */
	private Key referredKey = null;

	/**
     * Stores the fields contained in this database.
     * It is a field name (String) to Field mapping.
     * Cannot be null, have null keys or values.
     * Initialized with LinkedHashMap (because we need order).
     * Used by all the field accessor methods.
     */
	private Map fields = new LinkedHashMap();
	
    /**
     * Default constructor.
     * Doesn't do anything.
     */
	public Key() {
	}
	
	/**
	 * Gets the key name (keyName field)
	 * 
	 * @return the key name
	 */
	public String getKeyName() {
		return keyName;
	}

	/**
	 * Sets the key name (keyName field)
	 * 
	 * @param keyName the key name to set
	 * 
	 * @throws IllegalArgumentException if argument is null, empty or all spaces
	 */
	public void setKeyName(String keyName) throws IllegalArgumentException {
		this.keyName = keyName;
	}

	/**
	 * Gets the referred table (referredTable field)
	 * 
	 * @return the referred table
	 */
	public Table getReferredTable() {
		return referredTable;
	}

	/**
	 * Sets the referred table (referredTable field).
	 * 
	 * @param referredTable the referred table
	 */
	public void setReferredTable(Table referredTable) {
		this.referredTable = referredTable;
	}

	/**
	 * Gets the referred key (referredKey field)
	 * 
	 * @return the referred key
	 */
	public Key getReferredKey() {
		return referredKey;
	}

	/**
	 * Sets the referred key (referredKey field).
	 * 
	 * @param referredKey the referred key
	 */
	public void setReferredKey(Key referredKey) {
		this.referredKey = referredKey;
	}
	
	/**
	 * Adds a field to the list of fields.
	 * If a field with the same name exists, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Checks the mapping for field.getFieldName() and adds it to the mapping is not present.
	 * 
	 * @param field the field to add
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean addField(Field field) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Removes a field from the list of fields.
	 * If the field doesn't exist, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Delete the field.getFieldName() entry.
	 * 
	 * @param field the field to remove
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */	
	public boolean removeField(Field field) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears all fields.
	 * 
	 * Implementation details:
	 * 
	 * Clear the map.
	 */
	public void clearFields() {
	}
	
	/**
	 * Gets all fields.
	 * 
	 * Implementation details:
	 * 
	 * Make an array from the value set and return it.
	 * 
	 * @return the contained fields
	 */
	public Field[] getFields() {
		return null;
	}
	
	/**
	 * Gets a field by name.
	 * If no such field exist, null is returned
	 * 
	 * Implementation details:
	 * 
	 * Finds in the mapping the value for the name key.
	 * 
	 * @param name the field name
	 * @return the field with the given name or null if not found
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Field getField(String name) throws IllegalArgumentException {
		return null;
	}

}
