package com.topcoder.db.generation.xsd2ddl.data;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * This is a data class that models a table type.
 * The table type has a name and a list of fields.
 * 
 * All these attributes have appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class TableType {
	
	/**
	 * The name of the type.
	 * Can be null (in case of types directly nested in XSD) but cannot be empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String typeName = null;

	/**
     * Stores the fields contained in this database.
     * It is a field name (String) to Field mapping.
     * Cannot be null, have null keys or values.
     * Initialized with LinkedHashMap (because we need order).
     * Used by all the field accessor methods.
     */
    private Map fields = new LinkedHashMap();
    
    /**
     * Default constructor.
     * Doesn't do anything.
     */
    public TableType() {
    }

	/**
	 * Gets the type name (typeName field)
	 * 
	 * @return the type name
	 */
	public String getTypeName() {
		return typeName;
	}

	/**
	 * Sets the type name (typeName field)
	 * 
	 * @param typeName the type name to set (can be null)
	 * 
	 * @throws IllegalArgumentException if typeName is empty or all spaces
	 */
	public void setTypeName(String typeName) throws IllegalArgumentException {
		this.typeName = typeName;
	}
	
	/**
	 * Adds a field to the list of fields.
	 * If a field with the same name exists, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Checks the mapping for field.getFieldName() and adds it to the mapping is not present.
	 * 
	 * @param field the field to add
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean addField(Field field) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Removes a field from the list of fields.
	 * If the field doesn't exist, nothing is done and false is returned.
	 * 
	 * Implementation details:
	 * 
	 * Delete the field.getFieldName() entry.
	 * 
	 * @param field the field to remove
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */	
	public boolean removeField(Field field) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears all fields.
	 * 
	 * Implementation details:
	 * 
	 * Clear the map.
	 */
	public void clearFields() {
	}
	
	/**
	 * Gets all fields.
	 * 
	 * Implementation details:
	 * 
	 * Make an array from the value set and return it.
	 * 
	 * @return the contained fields
	 */
	public Field[] getFields() {
		return null;
	}
	
	/**
	 * Gets a field by name.
	 * If no such field exist, null is returned
	 * 
	 * Implementation details:
	 * 
	 * Finds in the mapping the value for the name key.
	 * 
	 * @param name the field name
	 * @return the field with the given name or null if not found
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Field getField(String name) throws IllegalArgumentException {
		return null;
	}

}
